﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using PerceptionComRpc;
using System.Net;
using System.Reflection;

namespace Perception.RPC.Demo
{
    public partial class MainForm : Form
     {
          private enum AcqState
          {
               Idle = 0,
               Pause = 1,
               Recording = 2
          }

          private struct Subscription
          {
               public int SubID;
               public string Name;
          };

          private AcqState m_bPreviousAcqState = AcqState.Idle;

          private const string TXT_Error_NoMainframes = "Perception has no hardware connected!\n\nAdvise: Load existing VWB or connect manually";
          private const string TXT_Error_NoChannels = "Can not find any channel, probable there is no hardware connected to Perception!\n\nAdvise: Load existing VWB or connect manually";
          private PerceptionCOM m_RPC = null;
          private MainframesClass m_Mainframes;
          private GroupsClass m_Groups;
          private double m_dBufferDuration = 0.2;     // Duration in seconds of the requested buffer
          private int m_nRequestNumberOfSamples = 1000;  // Number of samples in the requested buffer
          private int m_nMaxSegmentCount = 1;         // Maximum segment count used by the GetData() call
          private bool m_bConnected = false;
          private bool m_bTimerBusy = false;
          private bool m_bChannelSelectBusy = false;
          private bool m_bShowFrequency = true;
          string m_SelectedWaveform = "";
          private List<Subscription> m_Subscriptions = new List<Subscription>();
          private SubscriberThread m_SubscriberThread;

          private bool m_bIsIdle = false;
          private double m_dStartTime = double.NaN;
          private bool m_bAcqRateTooLow = false;
          private double m_dXStep = double.NaN;

          private int m_nTriggerMode = 0;

          public MainForm()
          {
               InitializeComponent();
               this.xyGraphControl1.ContextMenuStrip = this.contextMenuStrip1;
               lblFrequency.Text = "";
               toolStripStatusLabel1.Text = "Not connected";
               toolStripStatusLabel2.Text = "";
               toolStripStatusLabel3.Text = "";
               scalingTool1.Enabled = false;
               scalingTool2.Enabled = true;
               scalingTool1.xyDisplay = xyGraphControl1;
               scalingTool2.xyDisplay = xyGraphControl1;
               try
               {
                    richTextBox1.Rtf = Properties.Resources.Description_of_GENDAQ_RPC_Demo;
               }
               catch
               {
               }
               ShowVersionNumber();
               cbxRPCServers.SelectedIndex = 0;
               m_RPC = new PerceptionCOM();
               m_Mainframes = new MainframesClass();
               m_Groups = new GroupsClass();
               xyGraphControl1.SetPen(1, Color.Cyan);
               xyGraphControl1.SetPen(1, Color.Yellow);
               timer1.Interval = 500;
               UpdateAcqState();

               NewSubscriptionNeeded(2 * xyGraphControl1.XRangeInPixels, (xyGraphControl1.XTo - xyGraphControl1.XOrigin));

               m_SubscriberThread = new SubscriberThread();
               m_SubscriberThread.OnFinished += new EventHandler(m_SubscriberThread_Finished);
               m_SubscriberThread.OnNewDataBuffer += new SubscriberThread.OnNewDataBufferEventHandler(OnNewDataBuffer);
               StartDataRequest();
          }


          void OnNewDataBuffer(object sender, float[] aMinData, float[] aMaxData, int nPoints, double dStartTime, double dXStep, bool bAcqRateTooLow)
          {
               if (this.InvokeRequired)
               {
                    BeginInvoke(new SubscriberThread.OnNewDataBufferEventHandler(OnNewDataBuffer), sender, aMinData, aMaxData, nPoints, dStartTime, dXStep, bAcqRateTooLow);
                    return;
               }

               if (aMaxData == null) 
                    return;
               m_aMaxData = new float[aMaxData.Length];
               Array.Copy(aMaxData, m_aMaxData, aMaxData.Length);

               if (aMinData == null) 
                    return;
               m_aMinData = new float[aMinData.Length];
               Array.Copy(aMinData, m_aMinData, aMinData.Length);

               m_dXStep = dXStep;
               m_dStartTime = dStartTime;
               m_bAcqRateTooLow = bAcqRateTooLow;
          }

          public void ShowVersionNumber()
          {
               Assembly a = Assembly.GetExecutingAssembly();
               AssemblyName an = a.GetName();
               Version v = an.Version;

               lblVersion.Text = v.Major.ToString() + "." + v.Minor.ToString();
               lblBuild.Text = v.Build.ToString();
          }

          private void btnExit_Click(object sender, EventArgs e)
          {
               Close();
          }

          private void btnScan_Click(object sender, EventArgs e)
          {
               btnScan.Enabled = false;
               bool btnConnectEnabled = btnConnect.Enabled;
               try
               {

                    btnConnect.Enabled = false;
                    Application.DoEvents();
                    cbxRPCServers.Items.Clear();
                    Cursor.Current = Cursors.WaitCursor;
                    // And display them in a listbox on the form
                    string[] arStr = GetHostNames();

                    for (int i = 0; i < arStr.Length; i++)
                    {
                         cbxRPCServers.Items.Add(arStr[i]);
                    }
               }
               finally
               {
                    btnScan.Enabled = true;
                    btnConnect.Enabled = btnConnectEnabled;
                    Cursor.Current = Cursors.Default;
               }

               if (cbxRPCServers.Items.Count == 0)
               {
                    MessageBox.Show(this, "No RPC Perception servers found", "Scanning", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
               }
               cbxRPCServers.SelectedIndex = 0;
          }

          internal String[] GetHostNames()
          {
               int TimeOutMs = 2000;
               Array serverAddresses;
               List<string> hosts = new List<string>();
               hosts.Add("localhost");
               try
               {
                    // Get the ip addresses of all servers that reply within 2 seconds
                    m_RPC.ScanRPCServers(TimeOutMs, out serverAddresses);
               }
               catch
               {
                    return hosts.ToArray();
               }

               if (serverAddresses != null)
               {
                    foreach (string strIPAddress in serverAddresses)
                    {
                         try
                         {
                              IPHostEntry hostEntry = Dns.GetHostEntry(strIPAddress);
                              if (hostEntry == null)
                              {
                                   hosts.Add(strIPAddress);
                              }
                              else
                              {
                                   hosts.Add(hostEntry.HostName + " (" + strIPAddress + ")");
                              }
                         }
                         catch (Exception)
                         {
                              hosts.Add(strIPAddress);
                         }
                    }
               }
               List<string> uniqueHosts = hosts.Distinct().ToList();
               return uniqueHosts.ToArray();
          }

          private void btnConnect_Click(object sender, EventArgs e)
          {
               string strResult = "";
               string cAddress = cbxRPCServers.Text;

               int nPos = cAddress.IndexOf(" (");
               if (nPos > -1)
               {
                    cAddress = cAddress.Substring(0, nPos);
                    cAddress = cAddress.Trim();
               }
               Connect(cAddress, out strResult);
          }

          private void btnDisconnect_Click(object sender, EventArgs e)
          {
               m_Mainframes.Mainframes.Clear();

               if (m_bConnected)
               {
                    if (m_RPC != null)
                    {
                         m_RPC.DisconnectFromServer();
                    }
               }
               m_bConnected = false;
               UpdateUIConnectionStatus();
          }

          private void UpdateUIConnectionStatus()
          {
               if (m_bConnected)
               {
                    toolStripStatusLabel1.Text = "Connected to: " + cbxRPCServers.Text;
                    greenLED1.On = true;
                    timer1.Enabled = true;
                    GetAvailableWaveForms();
                    if (cbxChannels.Items.Count > 0)
                         cbxChannels.Enabled = true;
                    btnConnect.Enabled = false;
                    btnDisconnect.Enabled = true;
               }
               else
               {
                    toolStripStatusLabel1.Text = "Not connected";
                    greenLED1.On = false;
                    timer1.Enabled = false;
                    cbxChannels.Items.Clear();
                    cbxChannels.Enabled = false;
                    btnConnect.Enabled = true;
                    btnDisconnect.Enabled = false;
               }
               UpdateAcqState();
          }

          public bool Connect(string cAddress, out string Result)
          {
               Result = "";

               // Connect to RPC Server (Perception)
               try
               {
                    Cursor.Current = Cursors.WaitCursor;

                    btnConnect.Enabled = false;
                    m_RPC.SetServerAddress(cAddress);
                    m_RPC.ConnectToServer();

                    m_bConnected = true;
                    RetrieveHardwareNames();
               }
               catch (Exception)
               {
                    // RPC connection could not be setup.
                    Result = "Could not connect to Perception RPC Server";
                    m_bConnected = false;

                    UpdateUIConnectionStatus();
                    return false;
               }
               finally
               {
                    btnConnect.Enabled = !m_bConnected;
                    Cursor.Current = Cursors.Default;

               }
               UpdateUIConnectionStatus();
               return true;
          }

          internal bool RetrieveHardwareNames()
          {
               m_Mainframes.Mainframes.Clear();
               m_Groups.Groups.Clear();
               if (m_RPC == null) return false;
               if (!m_bConnected) return false;

               int nMainframeIndex = 1;
               string cMainframeIndex = "";
               string cMainframeName = "";
               int nRecorderIndex = 1;
               string cRecorderIndex = "";
               string cRecorderName = "";
               int nChannelIndex = 1;
               string cChannelIndex = "";
               string cChannelName = "";

               do
               {
                    cMainframeIndex = "$" + nMainframeIndex.ToString() + "$";
                    m_RPC.GetMainframeName(cMainframeIndex, out cMainframeName);
                    if (string.IsNullOrEmpty(cMainframeName))
                         break;
                    MainframeClass myNewMainframe = new MainframeClass(cMainframeName);
                    nRecorderIndex = 1;
                    do
                    {
                         cRecorderIndex = "$" + nRecorderIndex.ToString() + "$";
                         m_RPC.GetRecorderName(cMainframeName, cRecorderIndex, out cRecorderName);
                         if (string.IsNullOrEmpty(cRecorderName))
                              break;
                         RecorderClass myNewRecorder = new RecorderClass(cRecorderName);
                         nChannelIndex = 1;
                         do
                         {
                              cChannelIndex = "$" + nChannelIndex.ToString() + "$";
                              m_RPC.GetChannelName(cMainframeName, cRecorderName, cChannelIndex, out cChannelName);
                              if (string.IsNullOrEmpty(cChannelName))
                                   break;
                              ChannelClass myNewChannel = new ChannelClass(cChannelName);
                              myNewRecorder.AddChannel(myNewChannel);
                              nChannelIndex++;
                         } while (true);

                         myNewMainframe.AddRecorder(myNewRecorder);
                         nRecorderIndex++;
                    } while (true);

                    m_Mainframes.AddMainframe(myNewMainframe);
                    nMainframeIndex++;

               } while (true);

               Array arrayGroups;
               m_RPC.GetGroups(out arrayGroups);
               string[] aGroups = arrayGroups as string[];
               foreach (string cGroup in aGroups)
               {
                    GroupClass myNewGroup = new GroupClass(cGroup);

                    int nRecCount;
                    m_RPC.GetRecorderCountInGroup(cGroup, out nRecCount);

                    // we now loop through all recorders and check to ones which belong to this group
                    for (int i = 0; i < m_Mainframes.Mainframes.Count; i++)
                    {
                         MainframeClass myMainframe = m_Mainframes.Mainframes[i];
                         for (int j = 0; j < myMainframe.Recorders.Count; j++)
                         {
                              RecorderClass myRecorder = myMainframe.Recorders[j];
                              string cGroupName;
                              m_RPC.GetRecorderGroup(myMainframe.Name, myRecorder.Name, out cGroupName);
                              if (cGroupName == cGroup)
                                   myNewGroup.AddRecorder(myRecorder);
                         }
                    }
                    m_Groups.AddGroup(myNewGroup);
               }
               return (m_Mainframes.Mainframes.Count > 0);
          }

          private void UpdateAcqState()
          {
               AcqState NewAcqState = AcqState.Idle;
               bool bDisabled = false;
               if (m_RPC == null) bDisabled = true;
               if (!m_bConnected) bDisabled = true;

               try
               {
                    if (bDisabled)
                    {
                         LEDPreview.On = false;
                         LEDRecord.On = false;
                         LEDStopping.On = false;
                         LEDIdle.On = false;
                         acqControl1.SetAcqControl(false);
                         m_bIsIdle = true;
                         return;
                    }

                    int Run;
                    int OneShot;
                    int Stop;
                    int Pause;
                    int AcqIdle;
                    try
                    {

                         m_RPC.GetAcquisitionState(out Run, out  OneShot, out  Stop, out  Pause, out  AcqIdle);
                         m_bIsIdle = ((Run + OneShot + Stop + Pause) == 0);
                         if ((Run + OneShot + Stop + Pause + AcqIdle) == 0)
                         {
                              LEDPreview.On = false;
                              LEDRecord.On = false;
                              LEDStopping.On = false;
                              LEDIdle.On = false;
                              acqControl1.SetAcqControl(false);
                              return;
                         }

                         acqControl1.SetAcqControl(true);
                         if ((Run == 0) && (OneShot == 0) && (Stop == 0) && (Pause == 0) && (AcqIdle != 0))
                         {
                              LEDPreview.On = false;
                              LEDRecord.On = false;
                              LEDStopping.On = false;
                              LEDIdle.On = true;
                              return;
                         }
                         if ((Run != 0) || (OneShot != 0))
                         {
                              LEDPreview.On = false;
                              LEDRecord.On = true;
                              LEDStopping.On = false;
                              LEDIdle.On = false;
                              NewAcqState = AcqState.Recording;
                              return;
                         }
                         if ((Pause != 0))
                         {
                              LEDPreview.On = true;
                              LEDRecord.On = false;
                              LEDStopping.On = false;
                              LEDIdle.On = false;
                              NewAcqState = AcqState.Pause;
                              return;
                         }

                         LEDPreview.On = false;
                         LEDRecord.On = false;
                         LEDStopping.On = false;
                         LEDIdle.On = false;
                    }
                    catch
                    {
                    }
               }
               finally
               {
                    if (m_bPreviousAcqState != NewAcqState)
                    {
                         m_bPreviousAcqState = NewAcqState;
                    }
               }
          }

          private void timer1_Tick(object sender, EventArgs e)
          {
               if (m_bTimerBusy)
               {
                    return;
               }
               try
               {
                    m_bTimerBusy = true;
                    UpdateAcqState();
                    if (!m_bIsIdle)
                    {
                         if (m_SubscriberThread != null)
                         {
                              if (!m_SubscriberThread.Started)
                                   StartDataRequest();
                         }
                    }
                    ShowData();
               }
               finally
               {
                    scalingTool1.Enabled = xyGraphControl1.HasPoints();
                    m_bTimerBusy = false;
               }
          }

          private void acqControl1_OnPauseClick(object sender, EventArgs e)
          {
               if (!PerceptionReady()) return;
               if (m_bConnected)
                    m_RPC.Pause();
               UpdateAcqState();
               //ReSubscribe();
          }

          private bool PerceptionReady()
          {
               if (m_RPC == null) return false;
               if (!m_bConnected) return false;
               if (m_Mainframes.Mainframes.Count == 0)
                    RetrieveHardwareNames();
               if (m_Mainframes.Mainframes.Count == 0)
               {
                    MessageBox.Show(this, TXT_Error_NoMainframes, "RPC Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
               }
               return true;
          }

          private void acqControl1_OnRecordClick(object sender, EventArgs e)
          {
               if (!PerceptionReady()) return;
               m_RPC.Start();
               UpdateAcqState();

          }

          private void acqControl1_OnStopClick(object sender, EventArgs e)
          {
               if (!PerceptionReady()) return;
               m_RPC.Stop();
               UpdateAcqState();

               if (m_SubscriberThread != null)
               {
                    if (m_SubscriberThread.Started)
                    {
                         m_SubscriberThread.Cancel();
                    }
               }
          }

          private void acqControl1_OnTriggerClick(object sender, EventArgs e)
          {
               if (!PerceptionReady()) return;
               m_RPC.Trigger();
               UpdateAcqState();
          }

          internal void GetAvailableWaveForms()
          {
               try
               {
                    m_bChannelSelectBusy = true;
                    cbxChannels.Items.Clear();
                    Array aWaveformNames;
                    m_RPC.GetAvailableWaveforms(out aWaveformNames);

                    string[] strWaveformNames = aWaveformNames as string[];

                    for (int i = 0; i < strWaveformNames.Length; i++)
                    {
                         cbxChannels.Items.Add(strWaveformNames[i]);
                    }
                    if (strWaveformNames.Length == 0)
                    {
                         MessageBox.Show(this, TXT_Error_NoChannels, "RPC Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
                         return;
                    }
               }
               finally
               {
                    m_bChannelSelectBusy = false;
               }
               cbxChannels.SelectedIndex = 0;
          }


          private void SubScribeDescriptions()
          {
               m_SelectedWaveform = cbxChannels.Text;
               if (string.IsNullOrEmpty(m_SelectedWaveform))
                    return;
               try
               {
                    Subscription NewSubscription = new Subscription();
                    int nSubscriberID;
                    m_RPC.SubscribeLiveWaveform(m_SelectedWaveform, m_nRequestNumberOfSamples, m_dBufferDuration, m_nTriggerMode, 0, out nSubscriberID);
                    NewSubscription.Name = m_SelectedWaveform;
                    NewSubscription.SubID = nSubscriberID;
                    m_Subscriptions.Add(NewSubscription);
               }
               catch
               {
               }
          }

          private void UnSubScribeDescriptions()
          {
               if (m_RPC != null)
               {
                    if (m_bConnected)
                    {
                         if (m_Subscriptions.Count < 1) return;
                         for (int i = 0; i < m_Subscriptions.Count; i++)
                         {
                              try
                              {
                                   m_RPC.UnsubscribeLiveWaveform(m_Subscriptions[i].SubID);
                              }
                              catch
                              {
                              }
                         }
                    }
               }
               m_Subscriptions.Clear();
          }

          private void ReSubscribe()
          {
               if (!string.IsNullOrEmpty(m_SelectedWaveform))
               {
                    UnSubScribeDescriptions();
                    SubScribeDescriptions();
               }
          }

          private void cbxChannels_SelectedIndexChanged(object sender, EventArgs e)
          {
               if (m_bChannelSelectBusy)
                    return;

               try
               {
                    m_bChannelSelectBusy = true;
                    m_SelectedWaveform = cbxChannels.Text;
                    NewSubscriptionNeeded(2 * xyGraphControl1.XRangeInPixels, (xyGraphControl1.XTo - xyGraphControl1.XOrigin));
               }
               finally
               {
                    m_bChannelSelectBusy = false;
               }
          }

          internal static string ConvertTimeToString(double dValue)
          {
               string strFormat = "{0:0.000}";
               string cValue;

               if (dValue < 1E-9)
               {
                    // pico p
                    dValue = dValue * 1E12;
                    cValue = string.Format(strFormat, dValue) + "ps";
               }
               else if (dValue < 1E-6)
               {
                    // nano ɳ
                    dValue = dValue * 1E9;
                    cValue = string.Format(strFormat, dValue) + "ɳs";

               }
               else if (dValue < 1E-3)
               {
                    // micro µ
                    dValue = dValue * 1E6;
                    cValue = string.Format(strFormat, dValue) + "µs";
               }
               else if (dValue < 1)
               {
                    // milli m
                    dValue = dValue * 1E3;
                    cValue = string.Format(strFormat, dValue) + "ms";

               }
               else if (dValue < 10)
               {
                    cValue = string.Format(strFormat, dValue) + "s";
               }
               else if (dValue < 100)
               {
                    strFormat = "{0:0.00}";
                    cValue = string.Format(strFormat, dValue) + "s";
               }
               else
               {
                    strFormat = "{0:0.0}";
                    cValue = string.Format(strFormat, dValue) + "s";
               }

               return cValue;
          }

          private void xyGraphControl1_OnNewSubscriptionNeeded(object sender, int nPoints, float fTimeRange)
          {
               NewSubscriptionNeeded(nPoints, fTimeRange);
          }

          private void NewSubscriptionNeeded(int nPoints, float fTimeRange)
          {
               m_dBufferDuration = fTimeRange;
               m_nRequestNumberOfSamples = nPoints;
               string cRange = ConvertTimeToString(fTimeRange);
               lblTimeInterval.Text = "Time span = " + cRange;
               if (m_SubscriberThread != null)
               {
                    if (m_SubscriberThread.Started)
                    {
                         m_SubscriberThread.Cancel();
                         // Now first wait for finish event and from there a new StartDataRequest is done
                    }
                    else
                    {
                         StartDataRequest();
                    }
               }
          }

          private void StartDataRequest()
          {
               ReSubscribe();

               m_SubscriberThread.RPC = m_RPC;
               if (m_Subscriptions.Count > 0)
               {
                    m_nMaxSegmentCount = 10;
                    m_SubscriberThread.GetNewData(m_Subscriptions[0].SubID, m_nMaxSegmentCount, m_nRequestNumberOfSamples, m_dBufferDuration);
               }
          }

          float[] m_aMaxData;
          float[] m_aMinData;

          void m_SubscriberThread_Finished(object sender, EventArgs e)
          {
               if (m_bIsIdle)
               {
                    xyGraphControl1.Reset();
                    xyGraphControl1.Invalidate();
               }
               else
                    StartDataRequest();

          }

          void ShowData()
          {
               if (m_aMaxData != null)
               {
                    if (m_bAcqRateTooLow)
                         toolStripStatusLabel3.Text = "Perception acquisition rate too low for the requested data";
                    else
                         toolStripStatusLabel3.Text = "";
                    if (m_aMaxData.Length > 0)
                    {
                         xyGraphControl1.Reset();
                         if (double.IsNaN(m_dStartTime) || (m_dXStep == 0) || (double.IsNaN(m_dXStep)))
                         {
                              m_dXStep = 0.001;
                              xyGraphControl1.Reset();
                              xyGraphControl1.Invalidate();
                              UpdateAcqState();
                              return;
                         }

                         if (m_bShowFrequency)
                         {
                              int nSignalFrequency = (int)Math.Round(FrequencyCalculator.GetSignalFrequency(m_aMaxData, m_dXStep));
                              lblFrequency.Text = nSignalFrequency.ToString() + "Hz";
                         }

                         float fStartTime = (float)m_dStartTime;
                         xyGraphControl1.XOrigin = 0;
                         lblStart.Text = fStartTime.ToString();
                         float dEndTime = xyGraphControl1.XTo + fStartTime;
                         lblEnd.Text = dEndTime.ToString();
                         float dX1 = 0;

                         float fXStep = (float)m_dXStep;
                         float dY1;

                         double dXPixel = (xyGraphControl1.XTo - xyGraphControl1.XOrigin) / xyGraphControl1.XRangeInPixels;

                         if (m_dXStep > dXPixel)
                         {
                              // We do not have enough data the distance between the samples is larger then 1 pixel
                              // We use the average between the max and min value

                              if (m_dXStep > 10 * dXPixel) // If the step is more then 10 times the pixel step we show the individual points
                              {
                                   xyGraphControl1.DotShape = XYGraphControl.dotShape.BigDot;
                              }
                              else
                              {
                                   xyGraphControl1.DotShape = XYGraphControl.dotShape.Dot;
                              }

                              for (int j = 0; j < m_aMaxData.Length; j++)
                              {
                                   dY1 = (m_aMaxData[j] + m_aMinData[j]) / 2; // Average value between Min and Max
                                   xyGraphControl1.AddPoint(dX1, dY1);
                                   dX1 = dX1 + fXStep;
                              }
                         }
                         else
                         {
                              xyGraphControl1.DotShape = XYGraphControl.dotShape.Dot;
                              float fXHalfStep = fXStep / 2;
                              float dMaxVal = m_aMaxData[0];
                              float dMinVal = m_aMinData[0];
                              float dNextMaxVal;
                              float dNextMinVal;
                              float dY2;
                              for (int j = 1; j < m_aMaxData.Length; j++)
                              {
                                   dNextMaxVal = m_aMaxData[j];
                                   dNextMinVal = m_aMinData[j];

                                   if (dNextMinVal >= dMaxVal)
                                   {
                                        dY1 = dMinVal;
                                        dY2 = dMaxVal;
                                   }
                                   else
                                   {
                                        dY1 = dMaxVal;
                                        dY2 = dMinVal;
                                   }

                                   dMaxVal = dNextMaxVal;
                                   dMinVal = dNextMinVal;

                                   xyGraphControl1.AddPoint(dX1, dY1);
                                   xyGraphControl1.AddPoint(dX1 + fXHalfStep, dY2);
                                   dX1 = dX1 + fXStep;
                              }
                         }
                         xyGraphControl1.Invalidate();

                         m_dStartTime = double.NaN;
                         m_aMaxData = null;
                    }
               }
          }

          private void tabPage2_Resize(object sender, EventArgs e)
          {
               lblTimeInterval.Left = (xyGraphControl1.Width - lblTimeInterval.Width) / 2;
          }

          private void propertiesToolStripMenuItem_Click(object sender, EventArgs e)
          {
               XYGraphProperties Dialog = new XYGraphProperties();
               Dialog.XYGraphControl = xyGraphControl1;
               if (Dialog.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
               {

               }
          }

          private void toolStripMenuItemFrequency_Click(object sender, EventArgs e)
          {
               toolStripMenuItemFrequency.Checked = !toolStripMenuItemFrequency.Checked;
               m_bShowFrequency = toolStripMenuItemFrequency.Checked;
               lblFrequency.Visible = m_bShowFrequency;
          }

          #region Disposing
          private bool m_bDisposed = false;
          /// <summary>
          /// Procedure to do your cleanup, this function is called from within the Dispose() method.
          /// </summary>
          private void CleanUp()
          {
               if (m_bDisposed) return;

               try
               {
                    if (m_SubscriberThread != null)
                    {
                         if (m_SubscriberThread.Started)
                         {
                              m_SubscriberThread.Cancel();
                         }
                    }
                    m_RPC = null;
               }
               catch
               {
               }
               m_bDisposed = true;
          }
          #endregion

          private void monitorTriggerModeToolStripMenuItem_Click(object sender, EventArgs e)
          {
               TriggerModeSelection Dialog = new TriggerModeSelection();
               Dialog.TriggerMode = m_nTriggerMode;
               if (Dialog.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
               {
                   if ( m_nTriggerMode != Dialog.TriggerMode)
                   {
                        m_nTriggerMode = Dialog.TriggerMode;
                        NewSubscriptionNeeded(2 * xyGraphControl1.XRangeInPixels, (xyGraphControl1.XTo - xyGraphControl1.XOrigin));
                   }
               }
          }
     }
}
